@@ -199,38 +199,25 @@ mode_solver::mode_solver(int num_bands, double resolution[3], lattice lat, doubl
                          std::string epsilon_input_file, std::string mu_input_file, bool force_mu,
                          bool use_simple_preconditioner, vector3 grid_size, int eigensolver_nwork,
                          int eigensolver_block_size)
-    : num_bands(num_bands), target_freq(target_freq), tolerance(tolerance), mesh_size(mesh_size),
+    : num_bands(num_bands), resolution{resolution[0], resolution[1], resolution[2]},
+      target_freq(target_freq), tolerance(tolerance), mesh_size(mesh_size),
       negative_epsilon_ok(negative_epsilon_ok), epsilon_input_file(epsilon_input_file),
       mu_input_file(mu_input_file), force_mu(force_mu),
       use_simple_preconditioner(use_simple_preconditioner), grid_size(grid_size), nwork_alloc(0),
       eigensolver_nwork(eigensolver_nwork), eigensolver_block_size(eigensolver_block_size),
-      last_parity(-2), iterations(0), eigensolver_flops(flops), vol(0), mdata(NULL), mtdata(NULL),
-      curfield_band(0), freqs(num_bands), verbose(verbose), deterministic(deterministic),
-      kpoint_index(0), curfield(NULL), curfield_type('-'), eps(true) {
+      last_parity(-2), iterations(0), eigensolver_flops(flops), geometry_list{},
+      geometry_tree(NULL),  vol(0), R{}, G{},  mdata(NULL), mtdata(NULL),
+      curfield_band(0), H{}, Hblock{}, muinvH{}, W{}, freqs(num_bands), verbose(verbose),
+      deterministic(deterministic), kpoint_index(0), curfield(NULL), curfield_type('-'), eps(true) {
 
+  // See geom-ctl-io-defaults.c in libctl
   geometry_lattice = lat;
   dimensions = dims;
   ensure_periodicity = periodicity;
-  geometry_tree = NULL;
-  H.data = NULL;
-  Hblock.data = NULL;
-  muinvH.data = NULL;
-
-  for (int i = 0; i < MAX_NWORK; i++) {
-    W[i].data = NULL;
-  }
-
-  for (int i = 0; i < 3; ++i) {
-    this->resolution[i] = resolution[i];
-    for (int j = 0; j < 3; ++j) {
-      R[i][j] = 0.0;
-      G[i][j] = 0.0;
-    }
-  }
 
 #ifndef WITH_HERMITIAN_EPSILON
   meep_geom::medium_struct *m;
-  if (meep_geom::is_medium(_default_material, &m)) { meep_geom::check_offdiag(m); }
+  if (meep_geom::is_medium(_default_material, &m)) { m->check_offdiag_im_zero_or_abort(); }
 #else
   (void)_default_material;
 #endif
@@ -240,6 +227,7 @@ mode_solver::~mode_solver() {
   destroy_maxwell_data(mdata);
   destroy_maxwell_target_data(mtdata);
   destroy_geom_box_tree(geometry_tree);
+  clear_geometry_list();
   destroy_evectmatrix(H);
 
   for (int i = 0; i < nwork_alloc; ++i) {
@@ -534,7 +522,7 @@ void mode_solver::material_epsmu(meep_geom::material_type material, symmetric_ma
 #ifndef WITH_HERMITIAN_EPSILON
   if (md->which_subclass == meep_geom::material_data::MATERIAL_USER ||
       md->which_subclass == meep_geom::material_data::MATERIAL_FILE) {
-    meep_geom::check_offdiag(&md->medium);
+    md->medium.check_offdiag_im_zero_or_abort();
   }
 #endif
 
@@ -684,30 +672,30 @@ void mode_solver::get_material_pt(meep_geom::material_type &material, vector3 p)
 
 bool mode_solver::using_mu() { return mdata && mdata->mu_inv != NULL; }
 
-void mode_solver::init(int p, bool reset_fields, geometric_object_list geometry,
+void mode_solver::init(int p, bool reset_fields, geometric_object_list *geometry,
                        meep_geom::material_data *_default_material) {
   int have_old_fields = 0;
 
-  default_material = _default_material;
+  set_default_material(_default_material);
 
   n[0] = grid_size.x;
   n[1] = grid_size.y;
   n[2] = grid_size.z;
 
-  if (target_freq != 0.0 && mpb_verbosity >= 1) { meep::master_printf("Target frequency is %g\n", target_freq); }
+  if (target_freq != 0.0 && mpb_verbosity > 0) { meep::master_printf("Target frequency is %g\n", target_freq); }
 
   int true_rank = n[2] > 1 ? 3 : (n[1] > 1 ? 2 : 1);
   if (true_rank < dimensions) { dimensions = true_rank; }
   else if (true_rank > dimensions) {
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("WARNING: rank of grid is > dimensions.\n"
-                        "         setting extra grid dims. to 1.\n");
+                          "         setting extra grid dims. to 1.\n");
     // force extra dims to be 1
     if (dimensions <= 2) { n[2] = 1; }
     if (dimensions <= 1) { n[1] = 1; }
   }
 
-  if (mpb_verbosity >= 1) {
+  if (mpb_verbosity > 0) {
     meep::master_printf("Working in %d dimensions.\n", dimensions);
     meep::master_printf("Grid size is %d x %d x %d.\n", n[0], n[1], n[2]);
   }
@@ -721,7 +709,7 @@ void mode_solver::init(int p, bool reset_fields, geometric_object_list geometry,
       block_size = (num_bands - block_size - 1) / (-block_size);
       block_size = (num_bands + block_size - 1) / block_size;
     }
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("Solving for %d bands at a time.\n", block_size);
   }
   else {
@@ -762,19 +750,19 @@ void mode_solver::init(int p, bool reset_fields, geometric_object_list geometry,
     srand(314159); // * (rank + 1));
   }
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Creating Maxwell data...\n");
   mdata = create_maxwell_data(n[0], n[1], n[2], &local_N, &N_start, &alloc_N, block_size,
                               NUM_FFT_BANDS);
 
   if (target_freq != 0.0) { mtdata = create_maxwell_target_data(mdata, target_freq); }
 
-  init_epsilon(&geometry);
+  init_epsilon(geometry);
 
   if (check_maxwell_dielectric(mdata, 0)) { meep::abort("invalid dielectric function for MPB"); }
 
   if (!have_old_fields) {
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("Allocating fields...\n");
 
     int N = n[0] * n[1] * n[2];
@@ -809,19 +797,27 @@ void mode_solver::init(int p, bool reset_fields, geometric_object_list geometry,
   evectmatrix_flops = eigensolver_flops;
 }
 
-void mode_solver::init_epsilon(geometric_object_list *geometry) {
+void mode_solver::init_epsilon(geometric_object_list *geometry_in) {
+  // Persist geometry data and move it out of the input argument.
+  clear_geometry_list();
+  if (geometry_in->num_items && geometry_in->items) {
+    geometry_list.items = geometry_in->items;
+    geometry_list.num_items = geometry_in->num_items;
+    geometry_in->items = NULL;
+    geometry_in->num_items = 0;
+  }
   mpb_real no_size_x = geometry_lattice.size.x == 0 ? 1 : geometry_lattice.size.x;
   mpb_real no_size_y = geometry_lattice.size.y == 0 ? 1 : geometry_lattice.size.y;
   mpb_real no_size_z = geometry_lattice.size.z == 0 ? 1 : geometry_lattice.size.z;
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Mesh size is %d.\n", mesh_size);
 
   Rm.c0 = vector3_scale(no_size_x, geometry_lattice.basis.c0);
   Rm.c1 = vector3_scale(no_size_y, geometry_lattice.basis.c1);
   Rm.c2 = vector3_scale(no_size_z, geometry_lattice.basis.c2);
 
-  if (mpb_verbosity >= 1) {
+  if (mpb_verbosity > 0) {
     meep::master_printf("Lattice vectors:\n");
     meep::master_printf("     (%g, %g, %g)\n", Rm.c0.x, Rm.c0.y, Rm.c0.z);
     meep::master_printf("     (%g, %g, %g)\n", Rm.c1.x, Rm.c1.y, Rm.c1.z);
@@ -829,11 +825,11 @@ void mode_solver::init_epsilon(geometric_object_list *geometry) {
   }
 
   vol = fabs(matrix3x3_determinant(Rm));
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Cell volume = %g\n", vol);
 
   Gm = matrix3x3_inverse(matrix3x3_transpose(Rm));
-  if (mpb_verbosity >= 1) {
+  if (mpb_verbosity > 0) {
     meep::master_printf("Reciprocal lattice vectors (/ 2 pi):\n");
     meep::master_printf("     (%g, %g, %g)\n", Gm.c0.x, Gm.c0.y, Gm.c0.z);
     meep::master_printf("     (%g, %g, %g)\n", Gm.c1.x, Gm.c1.y, Gm.c1.z);
@@ -843,19 +839,20 @@ void mode_solver::init_epsilon(geometric_object_list *geometry) {
   matrix3x3_to_arr(R, Rm);
   matrix3x3_to_arr(G, Gm);
 
-  geom_fix_object_list(*geometry);
+  geom_fix_object_list(geometry_list);
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Geometric objects:\n");
   if (meep::am_master()) {
-    for (int i = 0; i < geometry->num_items; ++i) {
+    for (int i = 0; i < geometry_list.num_items; ++i) {
 
 #ifndef WITH_HERMITIAN_EPSILON
       meep_geom::medium_struct *mm;
-      if (meep_geom::is_medium(geometry->items[i].material, &mm)) { meep_geom::check_offdiag(mm); }
+      if (meep_geom::is_medium(geometry_list.items[i].material, &mm)) { mm->check_offdiag_im_zero_or_abort(); }
 #endif
 
-      display_geometric_object_info(5, geometry->items[i]);
+      if (mpb_verbosity > 0)
+        display_geometric_object_info(5, geometry_list.items[i]);
 
       // meep_geom::medium_struct *mm;
       // if (meep_geom::is_medium(geometry.items[i].material, &mm)) {
@@ -882,11 +879,12 @@ void mode_solver::init_epsilon(geometric_object_list *geometry) {
     b0.high.x += tmp_size.x / mdata->nx;
     b0.high.y += tmp_size.y / mdata->ny;
     b0.high.z += tmp_size.z / mdata->nz;
-    geometry_tree = create_geom_box_tree0(*geometry, b0);
+    destroy_geom_box_tree(geometry_tree);
+    geometry_tree = create_geom_box_tree0(geometry_list, b0);
   }
 
   if (verbose && meep::am_master()) {
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("Geometry object bounding box tree:\n");
     display_geom_box_tree(5, geometry_tree);
   }
@@ -894,14 +892,14 @@ void mode_solver::init_epsilon(geometric_object_list *geometry) {
   int tree_depth;
   int tree_nobjects;
   geom_box_tree_stats(geometry_tree, &tree_depth, &tree_nobjects);
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Geometric object tree has depth %d and %d object nodes"
                         " (vs. %d actual objects)\n",
-                        tree_depth, tree_nobjects, geometry->num_items);
+                        tree_depth, tree_nobjects, geometry_list.num_items);
 
   // restricted_tree = geometry_tree;
 
-  reset_epsilon(geometry);
+  reset_epsilon(&geometry_list);
 }
 
 void mode_solver::reset_epsilon(geometric_object_list *geometry) {
@@ -912,20 +910,22 @@ void mode_solver::reset_epsilon(geometric_object_list *geometry) {
   };
 
   if (!epsilon_input_file.empty()) {
-    default_material = meep_geom::make_file_material(epsilon_input_file.c_str());
+    meep_geom::material_type material = meep_geom::make_file_material(epsilon_input_file.c_str());
+    set_default_material(material);
+    material_free(material);
   }
 
   // TODO: support mu_input_file
   // if (!mu_input_file.empty()) {
   // }
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Initializing epsilon function...\n");
   set_maxwell_dielectric(mdata, mesh, R, G, dielectric_function, mean_epsilon_func,
                          static_cast<void *>(this));
 
   if (has_mu(geometry)) {
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("Initializing mu function...\n");
     eps = false;
     set_maxwell_mu(mdata, mesh, R, G, dielectric_function, mean_epsilon_func,
@@ -982,7 +982,7 @@ void mode_solver::set_parity(integer p) {
     meep::master_fprintf(stderr, "k vector incompatible with parity\n");
     exit(EXIT_FAILURE);
   }
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Solving for band polarization: %s.\n", parity_string(mdata));
 
   last_parity = p;
@@ -1001,7 +1001,7 @@ void mode_solver::set_kpoint_index(int i) { kpoint_index = i; }
 void mode_solver::randomize_fields() {
 
   if (!mdata) { return; }
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Initializing fields to random numbers...\n");
 
   for (int i = 0; i < H.n * H.p; ++i) {
@@ -1014,13 +1014,13 @@ void mode_solver::solve_kpoint(vector3 kvector) {
   // special handling of this k
   if (vector3_norm(kvector) < 1e-10) { kvector.x = kvector.y = kvector.z = 0; }
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("solve_kpoint (%g,%g,%g):\n", kvector.x, kvector.y, kvector.z);
 
   curfield_reset();
 
   if (num_bands == 0) {
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("  num-bands is zero, not solving for any bands\n");
     return;
   }
@@ -1032,7 +1032,7 @@ void mode_solver::solve_kpoint(vector3 kvector) {
 
   // If this is the first k point, print out a header line for the frequency
   // grep data.
-  if (mpb_verbosity >= 1) {
+  if (mpb_verbosity > 0) {
     if (!kpoint_index && meep::am_master()) {
       meep::master_printf("%sfreqs:, k index, k1, k2, k3, kmag/2pi", parity_string(mdata));
 
@@ -1054,7 +1054,7 @@ void mode_solver::solve_kpoint(vector3 kvector) {
 
   // TODO: Get flags from python
   int flags = EIGS_DEFAULT_FLAGS;
-  if (verbose || mpb_verbosity >= 1) { flags |= EIGS_VERBOSE; }
+  if (verbose || mpb_verbosity > 0) { flags |= EIGS_VERBOSE; }
 
   // Constant (zero frequency) bands at k=0 are handled specially, so remove
   // them from the solutions for the eigensolver.
@@ -1099,7 +1099,7 @@ void mode_solver::solve_kpoint(vector3 kvector) {
       evectmatrix_resize(&Hblock, num_bands - ib, 0);
     }
 
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("Solving for bands %d to %d...\n", ib + 1, ib + Hblock.p);
 
     constraints = NULL;
@@ -1158,14 +1158,14 @@ void mode_solver::solve_kpoint(vector3 kvector) {
 
     evect_destroy_constraints(constraints);
 
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf("Finished solving for bands %d to %d after %d iterations.\n", ib + 1,
                           ib + Hblock.p, num_iters);
 
     total_iters += num_iters * Hblock.p;
   }
 
-  if (num_bands - ib0 > Hblock.alloc_p && mpb_verbosity >= 1) {
+  if (num_bands - ib0 > Hblock.alloc_p && mpb_verbosity > 0) {
     meep::master_printf("Finished k-point with %g mean iterations/band.\n",
                         total_iters * 1.0 / num_bands);
   }
@@ -1202,17 +1202,17 @@ void mode_solver::solve_kpoint(vector3 kvector) {
 
   set_kpoint_index(kpoint_index + 1);
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("%sfreqs:, %d, %g, %g, %g, %g", parity_string(mdata), kpoint_index,
                         (double)k[0], (double)k[1], (double)k[2],
                         vector3_norm(matrix3x3_vector3_mult(Gm, kvector)));
 
   for (int i = 0; i < num_bands; ++i) {
     freqs[i] = negative_epsilon_ok ? eigvals[i] : sqrt(eigvals[i]);
-    if (mpb_verbosity >= 1)
+    if (mpb_verbosity > 0)
       meep::master_printf(", %g", freqs[i]);
   }
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("\n");
 
   eigensolver_flops = evectmatrix_flops;
@@ -1266,7 +1266,7 @@ void mode_solver::get_epsilon() {
   eps_mean /= N;
   eps_inv_mean = N / eps_inv_mean;
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("epsilon: %g-%g, mean %g, harm. mean %g, %g%% > 1, %g%% \"fill\"\n", eps_low,
                         eps_high, eps_mean, eps_inv_mean, (100.0 * fill_count) / N,
                         eps_high == eps_low ? 100.0
@@ -1322,7 +1322,7 @@ void mode_solver::get_mu() {
   eps_mean /= N;
   mu_inv_mean = N / mu_inv_mean;
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("mu: %g-%g, mean %g, harm. mean %g, %g%% > 1, %g%% \"fill\"\n", eps_low,
                         eps_high, eps_mean, mu_inv_mean, (100.0 * fill_count) / N,
                         eps_high == eps_low ? 100.0
@@ -1630,6 +1630,18 @@ double mode_solver::compute_field_energy_internal(mpb_real comp_sum[6]) {
   return energy_sum;
 }
 
+void mode_solver::clear_geometry_list() {
+  if (geometry_list.num_items && geometry_list.items) {
+    for(int i = 0; i < geometry_list.num_items; ++i) {
+      material_free((meep_geom::material_data *)geometry_list.items[i].material);
+      geometric_object_destroy(geometry_list.items[i]);
+    }
+    delete[] geometry_list.items;
+    geometry_list.items = NULL;
+    geometry_list.num_items = 0;
+  }
+}
+
 /* Replace curfield (either d or h) with the scalar energy density function,
    normalized to one.  While we're at it, compute some statistics about
    the relative strength of different field components.  Also return
@@ -1649,15 +1661,15 @@ std::vector<mpb_real> mode_solver::compute_field_energy() {
   mpb_real comp_sum[6];
   mpb_real energy_sum = compute_field_energy_internal(comp_sum);
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("%c-energy-components:, %d, %d", curfield_type, kpoint_index, curfield_band);
   for (int i = 0; i < 6; ++i) {
     comp_sum[i] /= (energy_sum == 0 ? 1 : energy_sum);
-    if (i % 2 == 1 && mpb_verbosity >= 1) {
+    if (i % 2 == 1 && mpb_verbosity > 0) {
       meep::master_printf(", %g", comp_sum[i] + comp_sum[i - 1]);
       }
   }
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("\n");
 
   /* The return value is a list of 7 items: the total energy,
@@ -2072,7 +2084,7 @@ void mode_solver::fix_field_phase() {
 
   ASSIGN_SCALAR(phase, SCALAR_RE(phase) * maxabs_sign, SCALAR_IM(phase) * maxabs_sign);
 
-  if (mpb_verbosity >= 1)
+  if (mpb_verbosity > 0)
     meep::master_printf("Fixing %c-field (band %d) phase by %g + %gi; "
                         "max ampl. = %g\n",
                         curfield_type, curfield_band, SCALAR_RE(phase), SCALAR_IM(phase), maxabs);
@@ -2655,7 +2667,7 @@ void map_data(mpb_real *d_in_re, int size_in_re, mpb_real *d_in_im, int size_in_
 #undef IN_INDEX
       }
 
-  if (verbose || mpb_verbosity >= 1) {
+  if (verbose || mpb_verbosity > 0) {
     meep::master_printf("real part range: %g .. %g\n", min_out_re, max_out_re);
     if (size_out_im > 0) meep::master_printf("imag part range: %g .. %g\n", min_out_im, max_out_im);
   }
